package net.wanji.utc.config.security;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.JwtParser;
import io.jsonwebtoken.Jwts;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.oauth2.client.OAuth2ClientContext;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.stereotype.Component;

import java.util.Map;

import static java.nio.charset.StandardCharsets.UTF_8;

/**
 * @Description: token 解析工具类，封装预置私钥读取方法及spring上下文中获取token方式
 * @Author mapabc
 * @Date 2020/6/17 11:00
 */
@Component
public class IdentityTokenHelper {

    @Value("${security.oauth2.resource.jwt.key-value}")
    private String signKey;

    @Autowired
    OAuth2ClientContext oAuth2ClientContext;

    public static final String TOKEN_EXTENSION_KEY = "jwt-ext";

    /**
     * @return io.jsonwebtoken.JwtParser
     * @Description: 获取jwt token解析器，封装私钥操作
     * 私钥自动读取security.oauth2.resource.jwt.key-value
     * 使用前置条件1：spring 应用扫描路径需要覆盖 com.wanji.identity.auth
     * <p>
     * JwtParser使用token body示例：IdentityTokenHelper#getJwtParser().parseClaimsJws(auth2AccessToken.getValue()).getBody();
     * @Author mapabc
     * @Date 2020/6/17 12:16
     **/
    public JwtParser getJwtParser() {
        if (StringUtils.isNotBlank(signKey)) {
            return Jwts.parser().setSigningKey(signKey.getBytes(UTF_8));
        }
        return null;
    }

    /**
     * @param key 键
     * @return java.lang.String 值，写入读取预置封装string
     * @Description: 根据key查询token扩展参数中的value
     * <p>
     * 前置条件1：使用com.wanji.identity.auth.OAuth2AccessTokenRestTemplate 获取token被传入扩展参数，使用方式可参考示例
     * @Author mapabc
     * @Date 2020/6/17 12:12
     * @see OAuth2AccessTokenRestTemplate#getTokenWithPasswordGrantType(String, String, Map)
     * 前置条件2：spring 应用扫描路径需要覆盖 com.wanji.identity.auth
     **/
    public String getValueByKeyFromTokenBodyExtension(String key) {
        if (StringUtils.isNotBlank(signKey)) {
            OAuth2AccessToken auth2AccessToken = oAuth2ClientContext.getAccessToken();
            if (null != auth2AccessToken) {
                Claims claims = Jwts.parser().setSigningKey(signKey.getBytes(UTF_8)).parseClaimsJws(auth2AccessToken.getValue()).getBody();
                if (null != claims && null != claims.get(TOKEN_EXTENSION_KEY)) {
                    if (claims.get(TOKEN_EXTENSION_KEY) instanceof Map) {
                        Map extension = (Map) claims.get(TOKEN_EXTENSION_KEY);
                        return (String) extension.getOrDefault(key, "");
                    }
                }
            }
        }
        return "";
    }

    public String getValueByKeyFromTokenBodyExtension(String tokenValue, String key) {
        if (StringUtils.isNotBlank(signKey)) {
            Claims claims = Jwts.parser().setSigningKey(signKey.getBytes(UTF_8)).parseClaimsJws(tokenValue).getBody();
            if (null != claims && null != claims.get(TOKEN_EXTENSION_KEY)) {
                if (claims.get(TOKEN_EXTENSION_KEY) instanceof Map) {
                    Map extension = (Map) claims.get(TOKEN_EXTENSION_KEY);
                    return (String) extension.getOrDefault(key, "");
                }
            }
        }
        return "";
    }
}
